#!/usr/bin/env python
# -*- coding: latin-1 -*-

"""Calculator module."""

# imports ####################################################################

import operator


# constants ##################################################################

ADD = "+"
SUB = "-"
MUL = "*"
DIV = "/"

NEG = "~"
EQU = "="
RST = "R"

OPS = { ADD:operator.add,
        SUB:operator.sub,
        MUL:operator.mul,
        DIV:operator.truediv }

HIGH_PRECEDENCE = [OPS[s] for s in [MUL, DIV]]


# calculator #################################################################

class Calculator:
	"""simple calculator functional core
	
	The calculator is implemented using 3 registers and 2 operators.
	ro and ro register store left and right operands,
	op stores the operation to perform with those operands.
	r1 and o1 are used to store intermediate results in 2 situations:
	- when a high precedence operation is started while a low one is not
	  finished ; and
	
	>>> c = Calculator()
	>>> c.interpret(2)
	2.0
	>>> c.interpret(ADD)
	2.0
	>>> c.interpret(3)
	3.0
	>>> c.interpret(MUL)
	3.0
	>>> c.interpret(5)
	5.0
	>>> c.interpret(ADD)
	17.0
	>>> c.interpret(1)
	1.0
	>>> c.interpret(EQU)
	18.0
	
	>>> c.interpret(RST)
	0.0
	>>> c.interpret(7)
	7.0
	>>> c.interpret(MUL)
	7.0
	>>> c.interpret(5)
	5.0
	>>> c.interpret(ADD)
	35.0
	>>> c.interpret(2)
	2.0
	>>> c.interpret(EQU)
	37.0
	
	- to store and recall the last operation perfomed when the user press
	  equal repeateadly.
	  
	>>> c.interpret(RST)
	0.0
	>>> c.interpret(2)
	2.0
	>>> c.interpret(ADD)
	2.0
	>>> c.interpret(3)
	3.0
	>>> c.interpret(EQU)
	5.0
	>>> c.interpret(EQU)
	8.0
	"""


	def __init__(self):
		self._reset()

		
	def _reset(self):
		self._r1 = None # register
		self._o1 = None # waiting operator
		self._lo = None # left operand
		self._op = None # operator
		self._ro = 0. # right operand
	

	def interpret(self, c):		
		try:
			if c in OPS:
				self._handle_operator(OPS[c])
			elif c == EQU:
				self._handle_equal()
			elif c == NEG:
				self._handle_neg()
			elif c == RST:
				self._reset()
			else:
				self._handle_number(c)
		
		except Exception, inst:
			self._ro = "Error"
			print inst
			
		return self._ro


	def _handle_operator(self, op):
		if self._op == None:
			# no unfinished calculation
			self._r1, self._o1, self._lo = None, None, self._ro
					
		elif self._op in HIGH_PRECEDENCE:
			# high precedence operation running
			self._ro = self._lo = self._op(self._lo, self._ro)
			if self._o1 != None and op not in HIGH_PRECEDENCE:
				self._ro = self._lo = self._o1(self._r1, self._lo)
				self._r1, self._o1 = None, None
					
		else:
			if op in HIGH_PRECEDENCE:
				self._r1, self._o1, self._lo = self._lo, self._op, self._ro
			else:
				self._ro = self._op(self._lo, self._ro)
						
		self._op = op
			
			
	def _handle_equal(self):
		if self._op == None:
			if self._o1 != None:
				# recalling last operation stored
				self._ro = self._o1(self._ro, self._r1)
					
		else:
			self._lo = self._op(self._lo, self._ro)
			if self._o1 != None:					
				self._lo = self._o1(self._r1, self._lo)
					
			# storing for repeated "="
			self._r1, self._o1 = self._ro, self._op
					
			# result
			self._op, self._ro = None, self._lo
	
	
	def _handle_neg(self):
		self._ro = -self._ro
		
	def _handle_number(self, n):
		self._ro = float(n)
			

def _test(argv=None):
	import doctest
	doctest.testmod()
	
	
if __name__ == "__main__":
	import sys
	sys.exit(_test())
	